from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

awsecrcrud_tests: list[RuleTest] = [
    RuleTest(
        name="Authorized account, unauthorized region",
        expected_result=True,
        log={
            "eventVersion": "1.04",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                "arn": "arn:aws:sts::123456789012:user/Mary_Major",
                "accountId": "123456789012",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "Mary_Major",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"}},
            },
            "eventTime": "2019-04-15T16:45:00Z",
            "eventSource": "ecr.amazonaws.com",
            "eventName": "PutImage",
            "awsRegion": "us-east-2",
            "sourceIPAddress": "203.0.113.12",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {
                "repositoryName": "testrepo",
                "imageTag": "latest",
                "registryId": "123456789012",
                "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
            },
            "responseElements": {
                "image": {
                    "repositoryName": "testrepo",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                    "registryId": "123456789012",
                    "imageId": {
                        "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                        "imageTag": "latest",
                    },
                },
            },
            "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
            "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
            "resources": [
                {"ARN": "arn:aws:ecr:us-east-2:123456789012:repository/testrepo", "accountId": "123456789012"},
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Authorized account",
        expected_result=False,
        log={
            "eventVersion": "1.04",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                "arn": "arn:aws:sts::123456789000:user/DeployRole",
                "accountId": "123456789000",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "Mary_Major",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"}},
            },
            "eventTime": "2019-04-15T16:45:00Z",
            "eventSource": "ecr.amazonaws.com",
            "eventName": "PutImage",
            "awsRegion": "us-east-2",
            "sourceIPAddress": "203.0.113.12",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {
                "repositoryName": "testrepo",
                "imageTag": "latest",
                "registryId": "123456789000",
                "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
            },
            "responseElements": {
                "image": {
                    "repositoryName": "testrepo",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                    "registryId": "123456789000",
                    "imageId": {
                        "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                        "imageTag": "latest",
                    },
                },
            },
            "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
            "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
            "resources": [
                {"ARN": "arn:aws:ecr:us-east-2:123456789000:repository/testrepo", "accountId": "123456789000"},
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789000",
        },
    ),
    RuleTest(
        name="Unauthorized Account",
        expected_result=True,
        log={
            "eventVersion": "1.04",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "AIDACKCEVSQ6C2EXAMPLE:account_name",
                "arn": "arn:aws:sts::123456789012:user/Mary_Major",
                "accountId": "123456789012",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "Mary_Major",
                "sessionContext": {"attributes": {"mfaAuthenticated": "false", "creationDate": "2019-04-15T16:42:14Z"}},
            },
            "eventTime": "2019-04-15T16:45:00Z",
            "eventSource": "ecr.amazonaws.com",
            "eventName": "PutImage",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "203.0.113.12",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {
                "repositoryName": "testrepo",
                "imageTag": "latest",
                "registryId": "123456789012",
                "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
            },
            "responseElements": {
                "image": {
                    "repositoryName": "testrepo",
                    "imageManifest": '{\n   "schemaVersion": 2,\n   "mediaType": "application/vnd.docker.distribution.manifest.v2+json",\n   "config": {\n      "mediaType": "application/vnd.docker.container.image.v1+json",\n      "size": 5543,\n      "digest": "sha256:000b9b805af1cdb60628898c9f411996301a1c13afd3dbef1d8a16ac6dbf503a"\n   },\n   "layers": [\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 43252507,\n         "digest": "sha256:3b37166ec61459e76e33282dda08f2a9cd698ca7e3d6bc44e6a6e7580cdeff8e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 846,\n         "digest": "sha256:504facff238fde83f1ca8f9f54520b4219c5b8f80be9616ddc52d31448a044bd"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 615,\n         "digest": "sha256:ebbcacd28e101968415b0c812b2d2dc60f969e36b0b08c073bf796e12b1bb449"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 850,\n         "digest": "sha256:c7fb3351ecad291a88b92b600037e2435c84a347683d540042086fe72c902b8a"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 168,\n         "digest": "sha256:2e3debadcbf7e542e2aefbce1b64a358b1931fb403b3e4aeca27cb4d809d56c2"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 37720774,\n         "digest": "sha256:f8c9f51ad524d8ae9bf4db69cd3e720ba92373ec265f5c390ffb21bb0c277941"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 30432107,\n         "digest": "sha256:813a50b13f61cf1f8d25f19fa96ad3aa5b552896c83e86ce413b48b091d7f01b"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 197,\n         "digest": "sha256:7ab043301a6187ea3293d80b30ba06c7bf1a0c3cd4c43d10353b31bc0cecfe7d"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 154,\n         "digest": "sha256:67012cca8f31dc3b8ee2305e7762fee20c250513effdedb38a1c37784a5a2e71"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 176,\n         "digest": "sha256:3bc892145603fffc9b1c97c94e2985b4cb19ca508750b15845a5d97becbd1a0e"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 183,\n         "digest": "sha256:6f1c79518f18251d35977e7e46bfa6c6b9cf50df2a79d4194941d95c54258d18"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:b7bcfbc2e2888afebede4dd1cd5eebf029bb6315feeaf0b56e425e11a50afe42"\n      },\n      {\n         "mediaType": "application/vnd.docker.image.rootfs.diff.tar.gzip",\n         "size": 212,\n         "digest": "sha256:2b220f8b0f32b7c2ed8eaafe1c802633bbd94849b9ab73926f0ba46cdae91629"\n      }\n   ]\n}',
                    "registryId": "123456789012",
                    "imageId": {
                        "imageDigest": "sha256:98c8b060c21d9adbb6b8c41b916e95e6307102786973ab93a41e8b86d1fc6d3e",
                        "imageTag": "latest",
                    },
                },
            },
            "requestID": "cf044b7d-5f9d-11e9-9b2a-95983139cc57",
            "eventID": "2bfd4ee2-2178-4a82-a27d-b12939923f0f",
            "resources": [
                {"ARN": "arn:aws:ecr:us-east-2:123456789012:repository/testrepo", "accountId": "123456789012"},
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSECRCRUD(Rule):
    id = "AWS.ECR.CRUD-prototype"
    display_name = "ECR CRUD Actions"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Configuration Required"]
    reports = {"CIS": ["3.12"], "MITRE ATT&CK": ["TA0005:T1525"]}
    default_severity = Severity.HIGH
    default_description = "Unauthorized ECR Create, Read, Update, or Delete event occurred."
    default_runbook = "https://docs.aws.amazon.com/AmazonECR/latest/userguide/logging-using-cloudtrail.html"
    default_reference = (
        "https://docs.aws.amazon.com/AmazonECR/latest/userguide/security-iam.html#security_iam_authentication"
    )
    summary_attributes = ["eventSource", "eventName", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    tests = awsecrcrud_tests
    ECR_CRUD_EVENTS = {
        "BatchCheckLayerAvailability",
        "BatchDeleteImage",
        "BatchGetImage",
        "CompleteLayerUpload",
        "CreateRepository",
        "DeleteRepository",
        "DeleteRepositoryPolicy",
        "GetAuthorizationToken",
        "GetDownloadUrlForLayer",
        "GetRepositoryPolicy",
        "InitiateLayerUpload",
        "PutImage",
        "SetRepositoryPolicy",
        "UploadLayerPart",
    }
    ALLOWED_ROLES = ["*DeployRole"]

    def rule(self, event):
        if event.get("eventSource") == "ecr.amazonaws.com" and event.get("eventName") in self.ECR_CRUD_EVENTS:
            for role in self.ALLOWED_ROLES:
                if fnmatch(deep_get(event, "userIdentity", "arn", default="unknown-arn"), role):
                    return False
            return True
        return False

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'arn', default='unknown-arn')}] performed ECR {event.get('eventName')} in [{event.get('recipientAccountId')} {event.get('awsRegion')}]."

    def dedup(self, event):
        return f"{deep_get(event, 'userIdentity', 'arn', default='unknown-arn')}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import duo_alert_context

duo_admin_mfa_restrictions_updated_tests: list[RuleTest] = [
    RuleTest(
        name="Admin MFA Update Event",
        expected_result=True,
        log={
            "action": "update_admin_factor_restrictions",
            "description": '{"allowed_factors": "Duo mobile passcodes, Hardware tokens, Duo push, Yubikey aes"}',
            "isotimestamp": "2022-02-21 21:48:06",
            "timestamp": "2022-02-21 21:48:06",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Login Event",
        expected_result=False,
        log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-789", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-06-30 19:45:37",
            "timestamp": "2021-06-30 19:45:37",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminMFARestrictionsUpdated(Rule):
    default_description = "Detects changes to allowed MFA factors administrators can use to log into the admin panel."
    display_name = "Duo Admin MFA Restrictions Updated"
    default_reference = "https://duo.com/docs/essentials-overview"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.MFA.Restrictions.Updated-prototype"
    tests = duo_admin_mfa_restrictions_updated_tests

    def rule(self, event):
        return event.get("action") == "update_admin_factor_restrictions"

    def title(self, event):
        return f"Duo Admin MFA Restrictions Updated by [{event.get('username', '<user_not_found>')}]"

    def alert_context(self, event):
        return duo_alert_context(event)



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardAWSVPCFlow(DataModel):
    id: str = "Standard.AWS.VPCFlow"
    display_name: str = "AWS VPCFlow"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_VPC_FLOW]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="destination_ip", path="dstAddr"),
        DataModelMapping(name="destination_port", path="dstPort"),
        DataModelMapping(name="source_ip", path="srcAddr"),
        DataModelMapping(name="source_port", path="srcPort"),
        DataModelMapping(name="user_agent", path="userAgent"),
        DataModelMapping(name="log_status", path="log-status"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_walk

gcp_service_account_access_denied_tests: list[RuleTest] = [
    RuleTest(
        name="service-account.access-denied-should-alert",
        expected_result=True,
        log={
            "insertid": "xxxxxxxxxxxx",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "test-no-perms@test-project-123456.iam.gserviceaccount.com",
                    "principalSubject": "serviceAccount:test-no-perms@test-project-123456.iam.gserviceaccount.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/test-project-123456/serviceAccounts/test-no-perms@test-project-123456.iam.gserviceaccount.com/keys/a0064fe0ef82b9e256b8b093d927ee842a19da34",
                },
                "authorizationInfo": [
                    {
                        "permission": "iam.serviceAccounts.create",
                        "resource": "projects/test-project-123456",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.admin.v1.CreateServiceAccount",
                "request": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.CreateServiceAccountRequest",
                    "account_id": "test123",
                    "name": "projects/test-project-123456",
                    "service_account": {},
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "google-cloud-sdk gcloud/431.0.0 command/gcloud.iam.service-accounts.create invocation-id/b2ea5dab8c9b4bff8bc15ab299dff79e environment/devshell environment-version/None client-os/LINUX client-os-ver/5.15.107 client-pltf-arch/x86_64 interactive/True from-script/False python/3.9.2 term/screen (Linux 5.15.107+),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-24T21:12:55.211301546Z"},
                },
                "resourceName": "projects/test-project-123456",
                "response": {"@type": "type.googleapis.com/google.iam.admin.v1.ServiceAccount"},
                "serviceName": "iam.googleapis.com",
                "status": {
                    "code": 7,
                    "details": [
                        {
                            "@type": "type.googleapis.com/google.rpc.ErrorInfo",
                            "domain": "iam.googleapis.com",
                            "metadata": {"permission": "iam.serviceAccounts.create"},
                            "reason": "IAM_PERMISSION_DENIED",
                        },
                    ],
                    "message": "Permission 'iam.serviceAccounts.create' denied on resource (or it may not exist).",
                },
            },
            "receivetimestamp": "2023-05-24 21:12:55.964",
            "resource": {
                "labels": {"email_id": "", "project_id": "test-project-123456", "unique_id": ""},
                "type": "service_account",
            },
            "severity": "ERROR",
            "timestamp": "2023-05-24 21:12:55.145",
        },
    ),
    RuleTest(
        name="service-account.access-grated-should-not-alert",
        expected_result=False,
        log={
            "insertId": "xxxxxxxxxxxx",
            "logName": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.serviceAccounts.create",
                        "resource": "projects/test-project-123456",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.admin.v1.CreateServiceAccount",
                "request": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.CreateServiceAccountRequest",
                    "account_id": "appengine-default",
                    "name": "projects/test-project-123456",
                    "service_account": {
                        "display_name": "App Engine default service account",
                        "email": "test-project-123456@appspot.gserviceaccount.com",
                    },
                },
                "requestMetadata": {
                    "callerIP": "private",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:27:42.510877536Z"},
                },
                "resourceName": "projects/test-project-123456",
                "response": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.ServiceAccount",
                    "display_name": "App Engine default service account",
                    "email": "test-project-123456@appspot.gserviceaccount.com",
                    "etag": "MDEwMjE5MjA=",
                    "name": "projects/test-project-123456/serviceAccounts/test-project-123456@appspot.gserviceaccount.com",
                    "oauth2_client_id": "114240096070638624820",
                    "project_id": "test-project-123456",
                    "unique_id": "114240096070638624820",
                },
                "serviceName": "iam.googleapis.com",
                "status": {"message": "OK"},
            },
            "receiveTimestamp": "2023-05-23 19:27:44.037",
            "resource": {
                "labels": {
                    "email_id": "test-project-123456@appspot.gserviceaccount.com",
                    "project_id": "test-project-123456",
                    "unique_id": "114240096070638624820",
                },
                "type": "service_account",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:27:42.492",
        },
    ),
]


@panther_managed
class GCPServiceAccountAccessDenied(Rule):
    dedup_period_minutes = 5
    threshold = 30
    display_name = "GCP Service Account Access Denied"
    id = "GCP.Service.Account.Access.Denied-prototype"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Service Account", "Access"]
    default_description = "This rule detects deletions of GCP Log Buckets or Sinks.\n"
    default_runbook = (
        "Ensure that the bucket or sink deletion was expected. Adversaries may do this to cover their tracks.\n"
    )
    default_reference = "https://cloud.google.com/iam/docs/service-account-overview"
    tests = gcp_service_account_access_denied_tests

    def rule(self, event):
        reason = deep_walk(event, "protoPayload", "status", "details", "reason", default="")
        return reason == "IAM_PERMISSION_DENIED"

    def title(self, event):
        actor = deep_walk(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed multiple requests resulting in [IAM_PERMISSION_DENIED]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_workspace_password_enforce_strong_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Workspace Admin Disabled Strong Password Enforcement",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "110111111111111111111"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:33:56.306000000",
                "uniqueQualifier": "-6444444444444444444",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_APPLICATION_SETTING",
            "parameters": {
                "APPLICATION_EDITION": "enterprise",
                "APPLICATION_NAME": "Security",
                "NEW_VALUE": "off",
                "OLD_VALUE": "on",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "Password Management - Enforce strong password",
            },
            "type": "APPLICATION_SETTINGS",
        },
    ),
    RuleTest(
        name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to READ_ONLY_ACCESS",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:06:26.303000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_CALENDAR_SETTING",
            "parameters": {
                "DOMAIN_NAME": "example.io",
                "NEW_VALUE": "READ_ONLY_ACCESS",
                "OLD_VALUE": "DEFAULT",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
            },
            "type": "CALENDAR_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteWorkspacePasswordEnforceStrongDisabled(Rule):
    id = "GSuite.Workspace.PasswordEnforceStrongDisabled-prototype"
    display_name = "GSuite Workspace Strong Password Enforcement Has Been Disabled"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_description = "A Workspace Admin Has Disabled The Enforcement Of Strong Passwords\n"
    default_reference = "https://support.google.com/a/answer/139399?hl=en"
    default_runbook = "Verify the intent of this Password Strength Setting Change. If intent cannot be verified, then a search on the actor's other activities is advised.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_workspace_password_enforce_strong_disabled_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName", default="").lower() != "admin":
            return False
        if all(
            [
                event.get("name", "") == "CHANGE_APPLICATION_SETTING",
                event.get("type", "") == "APPLICATION_SETTINGS",
                deep_get(event, "parameters", "NEW_VALUE", default="").lower() == "off",
                deep_get(event, "parameters", "SETTING_NAME", default="")
                == "Password Management - Enforce strong password",
            ],
        ):
            return True
        return False

    def title(self, event):
        return f"GSuite Workspace Strong Password Enforcement Has Been Disabled By [{deep_get(event, 'actor', 'email', default='<NO_ACTOR_FOUND>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_db_identity_provider_activity_tests: list[RuleTest] = [
    RuleTest(name="Random event", expected_result=False, log={"eventTypeName": "cat_jumped"}),
    RuleTest(
        name="FEDERATION_SETTINGS_CREATED",
        expected_result=True,
        log={"eventTypeName": "FEDERATION_SETTINGS_CREATED"},
    ),
    RuleTest(
        name="IDENTITY_PROVIDER_CREATED",
        expected_result=True,
        log={"eventTypeName": "IDENTITY_PROVIDER_CREATED"},
    ),
]


@panther_managed
class MongoDBIdentityProviderActivity(Rule):
    default_description = "Changes to identity provider settings are privileged activities that should be carefully audited.  Attackers may add or change IDP integrations to gain persistence to environments"
    display_name = "MongoDB Identity Provider Activity"
    default_severity = Severity.MEDIUM
    default_reference = "https://attack.mitre.org/techniques/T1556/007/"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.Identity.Provider.Activity-prototype"
    tests = mongo_db_identity_provider_activity_tests

    def rule(self, event):
        important_event_types = {
            "FEDERATION_SETTINGS_CREATED",
            "FEDERATION_SETTINGS_DELETED",
            "FEDERATION_SETTINGS_UPDATED",
            "IDENTITY_PROVIDER_CREATED",
            "IDENTITY_PROVIDER_UPDATED",
            "IDENTITY_PROVIDER_DELETED",
            "IDENTITY_PROVIDER_ACTIVATED",
            "IDENTITY_PROVIDER_DEACTIVATED",
            "IDENTITY_PROVIDER_JWKS_REVOKED",
            "OIDC_IDENTITY_PROVIDER_UPDATED",
            "OIDC_IDENTITY_PROVIDER_ENABLED",
            "OIDC_IDENTITY_PROVIDER_DISABLED",
        }
        return event.deep_get("eventTypeName") in important_event_types

    def title(self, event):
        target_username = event.get("targetUsername", "<USER_NOT_FOUND>")
        org_id = event.get("orgId", "<ORG_NOT_FOUND>")
        return f"MongoDB Atlas: User [{target_username}] roles changed in org [{org_id}]"

    def alert_context(self, event):
        return mongodb_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_new_app_detected_tests: list[RuleTest] = [
    RuleTest(
        name="New App",
        expected_result=True,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "approvalStatus": None,
                "creationTimestamp": 1698064423.0,
                "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "notes": "",
                "ownerId": None,
                "sensitivityLevel": None,
                "type": "ZAPIER",
            },
            "object": "APP",
            "old": None,
            "timestamp": 1698604061.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="App Updated",
        expected_result=False,
        log={
            "id": "c478966c-f927-411c-b919-179832d3d50c",
            "new": {
                "approvalStatus": "APPROVED",
                "creationTimestamp": 1698064423.0,
                "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "notes": "Last security audit: 16 January 2023.\n",
                "ownerId": "87569da6-fb7a-4df7-8ce2-246c14044911",
                "sensitivityLevel": "HIGH",
                "type": "ZAPIER",
            },
            "object": "APP",
            "old": {
                "approvalStatus": "UNDER_REVIEW",
                "creationTimestamp": 1698064423.0,
                "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                "notes": "Initial submission for review.\n",
                "ownerId": "87569da6-fb7a-4df7-8ce2-246c14044911",
                "sensitivityLevel": "MEDIUM",
                "type": "ZAPIER",
            },
            "timestamp": 1698604061.0,
            "type": "UPDATE",
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityNewAppDetected(Rule):
    id = "Push.Security.New.App.Detected-prototype"
    display_name = "Push Security New App Detected"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO
    tests = push_security_new_app_detected_tests

    def rule(self, event):
        if event.get("object") != "APP":
            return False
        if event.get("type") == "CREATE":
            return True
        return False

    def title(self, event):
        new_type = event.deep_get("new", "type")
        return f"New app in use: {new_type}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpiam_corporate_email_tests: list[RuleTest] = [
    RuleTest(
        name="Gmail account added",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {"principalEmail": "test@runpanther.com"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    "requestAttributes": {},
                    "destinationAttributes": {},
                },
                "serviceName": "cloudresourcemanager.googleapis.com",
                "methodName": "SetIamPolicy",
                "authorizationInfo": [
                    {
                        "resource": "projects/western-verve-123456",
                        "permission": "resourcemanager.projects.setIamPolicy",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                    {
                        "resource": "projects/western-verve-123456",
                        "permission": "resourcemanager.projects.setIamPolicy",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                ],
                "resourceName": "projects/western-verve-123456",
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {"action": "ADD", "role": "roles/viewer", "member": "user:username@gmail.com"},
                        ],
                    },
                },
                "request": {
                    "resource": "western-verve-123456",
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {
                        "bindings": [
                            {"members": ["user:user-two@gmail.com"], "role": "roles/appengine.serviceAdmin"},
                            {
                                "members": [
                                    "serviceAccount:service-951849100836@compute-system.iam.gserviceaccount.com",
                                ],
                                "role": "roles/compute.serviceAgent",
                            },
                            {
                                "role": "roles/editor",
                                "members": [
                                    "serviceAccount:951849100836-compute@developer.gserviceaccount.com",
                                    "serviceAccount:951849100836@cloudservices.gserviceaccount.com",
                                ],
                            },
                            {"members": ["user:test@runpanther.com"], "role": "roles/owner"},
                            {"members": ["user:user-two@gmail.com"], "role": "roles/pubsub.admin"},
                            {
                                "members": [
                                    "serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com",
                                ],
                                "role": "roles/pubsub.subscriber",
                            },
                            {
                                "members": [
                                    "serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com",
                                ],
                                "role": "roles/pubsub.viewer",
                            },
                            {
                                "role": "roles/resourcemanager.organizationAdmin",
                                "members": ["user:test@runpanther.com"],
                            },
                            {"members": ["user:username@gmail.com"], "role": "roles/viewer"},
                        ],
                        "etag": "BwWk8zJlg2o=",
                    },
                },
                "response": {
                    "etag": "BwWlp7rH6tY=",
                    "bindings": [
                        {"members": ["user:user-two@gmail.com"], "role": "roles/appengine.serviceAdmin"},
                        {
                            "members": ["serviceAccount:service-951849100836@compute-system.iam.gserviceaccount.com"],
                            "role": "roles/compute.serviceAgent",
                        },
                        {
                            "members": [
                                "serviceAccount:951849100836-compute@developer.gserviceaccount.com",
                                "serviceAccount:951849100836@cloudservices.gserviceaccount.com",
                            ],
                            "role": "roles/editor",
                        },
                        {"members": ["user:test@runpanther.com"], "role": "roles/owner"},
                        {"role": "roles/pubsub.admin", "members": ["user:user-two@gmail.com"]},
                        {
                            "role": "roles/pubsub.subscriber",
                            "members": ["serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com"],
                        },
                        {
                            "members": ["serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com"],
                            "role": "roles/pubsub.viewer",
                        },
                        {"members": ["user:test@runpanther.com"], "role": "roles/resourcemanager.organizationAdmin"},
                        {"members": ["user:username@gmail.com"], "role": "roles/viewer"},
                    ],
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                },
            },
            "insertId": "mrbji0dal80",
            "resource": {"type": "project", "labels": {"project_id": "western-verve-123456"}},
            "timestamp": "2020-05-15T03:51:35.019Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "receiveTimestamp": "2020-05-15T03:51:35.977314225Z",
        },
    ),
    RuleTest(
        name="Runpanther account added",
        expected_result=False,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {"principalEmail": "test@runpanther.com"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    "requestAttributes": {},
                    "destinationAttributes": {},
                },
                "serviceName": "cloudresourcemanager.googleapis.com",
                "methodName": "SetIamPolicy",
                "authorizationInfo": [
                    {
                        "resource": "projects/western-verve-123456",
                        "permission": "resourcemanager.projects.setIamPolicy",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                    {
                        "resource": "projects/western-verve-123456",
                        "permission": "resourcemanager.projects.setIamPolicy",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                ],
                "resourceName": "projects/western-verve-123456",
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {"action": "ADD", "role": "roles/viewer", "member": "user:username@runpanther.com"},
                        ],
                    },
                },
                "request": {
                    "resource": "western-verve-123456",
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {
                        "bindings": [
                            {"members": ["user:user-two@gmail.com"], "role": "roles/appengine.serviceAdmin"},
                            {
                                "members": [
                                    "serviceAccount:service-951849100836@compute-system.iam.gserviceaccount.com",
                                ],
                                "role": "roles/compute.serviceAgent",
                            },
                            {
                                "role": "roles/editor",
                                "members": [
                                    "serviceAccount:951849100836-compute@developer.gserviceaccount.com",
                                    "serviceAccount:951849100836@cloudservices.gserviceaccount.com",
                                ],
                            },
                            {"members": ["user:test@runpanther.com"], "role": "roles/owner"},
                            {"members": ["user:user-two@gmail.com"], "role": "roles/pubsub.admin"},
                            {
                                "members": [
                                    "serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com",
                                ],
                                "role": "roles/pubsub.subscriber",
                            },
                            {
                                "members": [
                                    "serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com",
                                ],
                                "role": "roles/pubsub.viewer",
                            },
                            {
                                "role": "roles/resourcemanager.organizationAdmin",
                                "members": ["user:test@runpanther.com"],
                            },
                            {"members": ["user:username@gmail.com"], "role": "roles/viewer"},
                        ],
                        "etag": "BwWk8zJlg2o=",
                    },
                },
                "response": {
                    "etag": "BwWlp7rH6tY=",
                    "bindings": [
                        {"members": ["user:user-two@gmail.com"], "role": "roles/appengine.serviceAdmin"},
                        {
                            "members": ["serviceAccount:service-951849100836@compute-system.iam.gserviceaccount.com"],
                            "role": "roles/compute.serviceAgent",
                        },
                        {
                            "members": [
                                "serviceAccount:951849100836-compute@developer.gserviceaccount.com",
                                "serviceAccount:951849100836@cloudservices.gserviceaccount.com",
                            ],
                            "role": "roles/editor",
                        },
                        {"members": ["user:test@runpanther.com"], "role": "roles/owner"},
                        {"role": "roles/pubsub.admin", "members": ["user:user-two@gmail.com"]},
                        {
                            "role": "roles/pubsub.subscriber",
                            "members": ["serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com"],
                        },
                        {
                            "members": ["serviceAccount:pubsub-reader@western-verve-123456.iam.gserviceaccount.com"],
                            "role": "roles/pubsub.viewer",
                        },
                        {"members": ["user:test@runpanther.com"], "role": "roles/resourcemanager.organizationAdmin"},
                        {"members": ["user:username@gmail.com"], "role": "roles/viewer"},
                    ],
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                },
            },
            "insertId": "mrbji0dal80",
            "resource": {"type": "project", "labels": {"project_id": "western-verve-123456"}},
            "timestamp": "2020-05-15T03:51:35.019Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "receiveTimestamp": "2020-05-15T03:51:35.977314225Z",
        },
    ),
]


@panther_managed
class GCPIAMCorporateEmail(Rule):
    id = "GCP.IAM.CorporateEmail-prototype"
    display_name = "GCP Corporate Email Not Used"
    dedup_period_minutes = 720
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Identity & Access Management", "Persistence:Create Account"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"], "CIS": ["1.1"]}
    default_severity = Severity.LOW
    default_description = "A Gmail account is being used instead of a corporate email"
    default_runbook = "Remove the user"
    default_reference = "https://cloud.google.com/iam/docs/service-account-overview"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = gcpiam_corporate_email_tests

    def rule(self, event):
        if deep_get(event, "protoPayload", "methodName") != "SetIamPolicy":
            return False
        service_data = deep_get(event, "protoPayload", "serviceData")
        if not service_data:
            return False
        # Reference: bit.ly/2WsJdZS
        binding_deltas = deep_get(service_data, "policyDelta", "bindingDeltas")
        if not binding_deltas:
            return False
        for delta in binding_deltas:
            if delta.get("action") != "ADD":
                continue
            if delta.get("member", "").endswith("@gmail.com"):
                return True
        return False

    def title(self, event):
        return f"A GCP IAM account has been created with a Gmail email in {deep_get(event, 'resource', 'labels', 'project_id', default='<UNKNOWN_PROJECT>')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config

g_suite_doc_ownership_transfer_tests: list[RuleTest] = [
    RuleTest(
        name="Ownership Transferred Within Organization",
        expected_result=False,
        log={
            "id": {"applicationName": "admin"},
            "name": "TRANSFER_DOCUMENT_OWNERSHIP",
            "parameters": {"NEW_VALUE": "homer.simpson@example.com"},
        },
    ),
    RuleTest(
        name="Document Transferred to External User",
        expected_result=True,
        log={
            "id": {"applicationName": "admin"},
            "name": "TRANSFER_DOCUMENT_OWNERSHIP",
            "parameters": {"NEW_VALUE": "monty.burns@badguy.com"},
        },
    ),
]


@panther_managed
class GSuiteDocOwnershipTransfer(Rule):
    id = "GSuite.DocOwnershipTransfer-prototype"
    display_name = "GSuite Document External Ownership Transfer"
    enabled = False
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Configuration Required", "Collection:Data from Information Repositories"]
    reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    default_severity = Severity.LOW
    default_description = "A GSuite document's ownership was transferred to an external party.\n"
    default_reference = (
        "https://support.google.com/drive/answer/2494892?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    )
    default_runbook = "Verify that this document did not contain sensitive or private company information.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_doc_ownership_transfer_tests
    GSUITE_TRUSTED_OWNERSHIP_DOMAINS = {"@" + domain for domain in config.GSUITE_TRUSTED_OWNERSHIP_DOMAINS}

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "admin":
            return False
        if bool(event.get("name") == "TRANSFER_DOCUMENT_OWNERSHIP"):
            new_owner = deep_get(event, "parameters", "NEW_VALUE", default="<UNKNOWN USER>")
            return bool(new_owner) and (not any(new_owner.endswith(x) for x in self.GSUITE_TRUSTED_OWNERSHIP_DOMAINS))
        return False



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

awss3_bucket_policy_modified_tests: list[RuleTest] = [
    RuleTest(
        name="S3 Bucket Policy Modified",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "PutBucketAcl",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "host": ["bucket.s3.us-west-2.amazonaws.com"],
                "bucketName": "bucket",
                "acl": [""],
                "x-amz-acl": ["private"],
            },
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "AuthenticationMethod": "AuthHeader",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="S3 Bucket Policy Modified Error Response",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "errorCode": "AccessDenied",
            "eventName": "PutBucketAcl",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "host": ["bucket.s3.us-west-2.amazonaws.com"],
                "bucketName": "bucket",
                "acl": [""],
                "x-amz-acl": ["private"],
            },
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "AuthenticationMethod": "AuthHeader",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="S3 Bucket Policy Not Modified",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "s3.amazonaws.com",
            "eventName": "GetBucketPolicy",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "host": ["bucket.s3.us-west-2.amazonaws.com"],
                "bucketName": "bucket",
                "policy": [""],
            },
            "responseElements": None,
            "additionalEventData": {
                "SignatureVersion": "SigV4",
                "CipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "AuthenticationMethod": "AuthHeader",
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSS3BucketPolicyModified(Rule):
    id = "AWS.S3.BucketPolicyModified-prototype"
    display_name = "AWS S3 Bucket Policy Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "Exfiltration:Exfiltration Over Web Service"]
    reports = {"CIS": ["3.8"], "MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "An S3 Bucket was modified.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-s3-bucket-policy-modified"
    default_reference = "https://docs.aws.amazon.com/AmazonS3/latest/dev/using-iam-policies.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "p_any_aws_arns"]
    tests = awss3_bucket_policy_modified_tests
    # API calls that are indicative of KMS CMK Deletion
    S3_POLICY_CHANGE_EVENTS = {
        "PutBucketAcl",
        "PutBucketPolicy",
        "PutBucketCors",
        "PutBucketLifecycle",
        "PutBucketReplication",
        "DeleteBucketPolicy",
        "DeleteBucketCors",
        "DeleteBucketLifecycle",
        "DeleteBucketReplication",
    }

    def rule(self, event):
        return event.get("eventName") in self.S3_POLICY_CHANGE_EVENTS and aws_cloudtrail_success(event)

    def title(self, event):
        return f"S3 bucket modified by [{deep_get(event, 'userIdentity', 'arn')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsrds_snapshot_shared_tests: list[RuleTest] = [
    RuleTest(
        name="Snapshot shared with another account",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
                        "accountId": "123456789012",
                        "userName": "AWSReservedSSO_DevAdmin_635426549a280cc6",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-12T19:43:57Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-12T20:12:22Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "ModifyDBSnapshotAttribute",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "68319f60-9dec-43b2-9702-de3a08c9d8a3",
            "requestParameters": {
                "dBSnapshotIdentifier": "exfiltration",
                "attributeName": "restore",
                "valuesToAdd": ["193672423079"],
            },
            "responseElements": {
                "dBSnapshotIdentifier": "exfiltration",
                "dBSnapshotAttributes": [{"attributeName": "restore", "attributeValues": ["193672423079"]}],
            },
            "requestID": "b7f91314-eb8b-4be5-995d-6b97d70dfb3b",
            "eventID": "86581591-0f39-4eae-9a8d-b2224a3c91fa",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
    RuleTest(
        name="Snapshot shared with no accounts",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "AROA2DFDF0C1FDFCAD2B2:fake.user",
                "arn": "arn:aws:sts::123456789012:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/fake.user",
                "accountId": "123456789012",
                "accessKeyId": "ASIAFFA5AFEC02FFCD8ED",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "AROA2DFDF0C1FDFCAD2B2",
                        "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
                        "accountId": "123456789012",
                        "userName": "AWSReservedSSO_DevAdmin_635426549a280cc6",
                    },
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-12-12T19:43:57Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-12-12T20:12:22Z",
            "eventSource": "rds.amazonaws.com",
            "eventName": "ModifyDBSnapshotAttribute",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "68319f60-9dec-43b2-9702-de3a08c9d8a3",
            "requestParameters": {
                "dBSnapshotIdentifier": "exfiltration",
                "attributeName": "restore",
                "valuesToAdd": [],
            },
            "responseElements": {
                "dBSnapshotIdentifier": "exfiltration",
                "dBSnapshotAttributes": [{"attributeName": "restore", "attributeValues": []}],
            },
            "requestID": "b7f91314-eb8b-4be5-995d-6b97d70dfb3b",
            "eventID": "86581591-0f39-4eae-9a8d-b2224a3c91fa",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.3",
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "rds.us-west-2.amazonaws.com",
            },
        },
    ),
]


@panther_managed
class AWSRDSSnapshotShared(Rule):
    id = "AWS.RDS.SnapshotShared-prototype"
    display_name = "AWS RDS Snapshot Shared"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration", "Transfer Data to Cloud Account"]
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = (
        "An RDS snapshot was shared with another account. This could be an indicator of exfiltration.\n"
    )
    default_runbook = "Ensure that the snapshot was shared intentionally and with an approved account. If not, remove the snapshot and quarantine the compromised IAM user.\n"
    default_reference = "https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_ShareSnapshot.html"
    summary_attributes = ["eventSource", "recipientAccountId", "awsRegion", "p_any_aws_arns"]
    tests = awsrds_snapshot_shared_tests

    def rule(self, event):
        if all(
            [
                event.get("eventSource", "") == "rds.amazonaws.com",
                event.get("eventName", "") == "ModifyDBSnapshotAttribute"
                or event.get("eventName", "") == "ModifyDBClusterSnapshotAttribute",
                event.deep_get("requestParameters", "attributeName") == "restore",
            ],
        ):
            current_account_id = event.deep_get("userIdentity", "accountId", default="")
            shared_account_ids = event.deep_get("requestParameters", "valuesToAdd", default=[])
            if shared_account_ids:
                return any(account_id for account_id in shared_account_ids if account_id != current_account_id)
            return False
        return False

    def title(self, event):
        account_id = event.get("recipientAccountId", default="<ACCOUNT_ID_NOT_FOUND>")
        rds_instance_id = event.deep_get(
            "responseElements",
            "dBInstanceIdentifier",
            default="<DB_INSTANCE_ID_NOT_FOUND>",
        )
        return f"RDS Snapshot Shared in [{account_id}] for RDS instance [{rds_instance_id}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_cloudflare_helpers import cloudflare_fw_alert_context

cloudflare_firewall_l7_d_do_s_tests: list[RuleTest] = [
    RuleTest(
        name="Traffic Marked as L7DDoS",
        expected_result=True,
        log={
            "Action": "skip",
            "ClientASN": 55836,
            "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
            "ClientCountry": "in",
            "ClientIP": "127.0.0.1",
            "ClientRequestHost": "example.com",
            "ClientRequestMethod": "GET",
            "ClientRequestPath": "/main.php",
            "ClientRequestProtocol": "HTTP/1.1",
            "ClientRequestQuery": "",
            "ClientRequestScheme": "http",
            "ClientRequestUserAgent": "Fuzz Faster U Fool v1.3.1-dev",
            "Datetime": "2022-05-10 06:36:57",
            "EdgeColoCode": "DEL",
            "EdgeResponseStatus": 403,
            "Kind": "firewall",
            "MatchIndex": 0,
            "Metadata": {"dos-source": "dosd-edge"},
            "OriginResponseStatus": 0,
            "OriginatorRayID": "00",
            "RayID": "7090a9da88e333d8",
            "RuleID": "ed651449c4a54f4b99c6e3bf863134d5",
            "Source": "l7ddos",
        },
    ),
    RuleTest(
        name="Traffic Marked as L7DDoS but blocked ( INFO level alert )",
        expected_result=True,
        log={
            "Action": "block",
            "ClientASN": 55836,
            "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
            "ClientCountry": "in",
            "ClientIP": "127.0.0.1",
            "ClientRequestHost": "example.com",
            "ClientRequestMethod": "GET",
            "ClientRequestPath": "/main.php",
            "ClientRequestProtocol": "HTTP/1.1",
            "ClientRequestQuery": "",
            "ClientRequestScheme": "http",
            "ClientRequestUserAgent": "Fuzz Faster U Fool v1.3.1-dev",
            "Datetime": "2022-05-10 06:36:57",
            "EdgeColoCode": "DEL",
            "EdgeResponseStatus": 403,
            "Kind": "firewall",
            "MatchIndex": 0,
            "Metadata": {"dos-source": "dosd-edge"},
            "OriginResponseStatus": 0,
            "OriginatorRayID": "00",
            "RayID": "7090a9da88e333d8",
            "RuleID": "ed651449c4a54f4b99c6e3bf863134d5",
            "Source": "l7ddos",
        },
    ),
    RuleTest(
        name="Traffic Not Marked as L7DDoS",
        expected_result=False,
        log={
            "Action": "block",
            "ClientASN": 55836,
            "ClientASNDescription": "RELIANCEJIO-IN Reliance Jio Infocomm Limited",
            "ClientCountry": "in",
            "ClientIP": "127.0.0.1",
            "ClientRequestHost": "example.com",
            "ClientRequestMethod": "GET",
            "ClientRequestPath": "/main.php",
            "ClientRequestProtocol": "HTTP/1.1",
            "ClientRequestQuery": "",
            "ClientRequestScheme": "http",
            "ClientRequestUserAgent": "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
            "Datetime": "2022-05-10 06:36:57",
            "EdgeColoCode": "DEL",
            "EdgeResponseStatus": 403,
            "Kind": "firewall",
            "MatchIndex": 0,
            "Metadata": {"dos-source": "dosd-edge"},
            "OriginResponseStatus": 0,
            "OriginatorRayID": "00",
            "RayID": "708174c00f61faa8",
            "RuleID": "e35c9a670b864a3ba0203ffb1bc977d1",
            "Source": "firewallmanaged",
        },
    ),
]


@panther_managed
class CloudflareFirewallL7DDoS(Rule):
    id = "Cloudflare.Firewall.L7DDoS-prototype"
    display_name = "Cloudflare L7 DDoS"
    log_types = [LogType.CLOUDFLARE_FIREWALL]
    tags = ["Cloudflare", "Variable Severity"]
    default_severity = Severity.MEDIUM
    default_description = "Layer 7 Distributed Denial of Service (DDoS) detected"
    default_runbook = "Inspect and monitor internet-facing services for potential outages"
    default_reference = "https://www.cloudflare.com/en-gb/learning/ddos/application-layer-ddos-attack/"
    threshold = 100
    summary_attributes = ["Action", "ClientCountry", "ClientIP", "ClientRequestUserAgent"]
    tests = cloudflare_firewall_l7_d_do_s_tests

    def rule(self, event):
        return event.get("Source", "") == "l7ddos"

    def title(self, event):
        return f"Cloudflare: Detected L7 DDoSfrom [{event.get('ClientIP', '<NO_CLIENTIP>')}] to [{event.get('ClientRequestHost', '<NO_REQ_HOST>')}] and took action [{event.get('Action', '<NO_ACTION>')}]"

    def alert_context(self, event):
        return cloudflare_fw_alert_context(event)

    def severity(self, event):
        if event.get("Action", "") == "block":
            return "Info"
        return "Medium"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

panther_sensitive_role_tests: list[RuleTest] = [
    RuleTest(
        name="Admin Role Created",
        expected_result=True,
        log={
            "actionName": "CREATE_USER_ROLE",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "logTypeAccessKind": "DENY_ALL",
                        "name": "New Admins",
                        "permissions": ["GeneralSettingsModify", "GeneralSettingsRead", "SummaryRead"],
                    },
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "1111111"},
                "id": "11111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
            "pantherVersion": "1.2.3",
            "sourceIP": "1.2.3.4",
            "timestamp": "2022-04-27 20:47:09.425",
        },
    ),
    RuleTest(
        name="Non-Admin Role Created",
        expected_result=False,
        log={
            "actionName": "CREATE_USER_ROLE",
            "actionParams": {
                "dynamic": {
                    "input": {"logTypeAccessKind": "DENY_ALL", "name": "New Admins", "permissions": ["SummaryRead"]},
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "1111111"},
                "id": "11111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
            "pantherVersion": "1.2.3",
            "sourceIP": "1.2.3.4",
            "timestamp": "2022-04-27 20:47:09.425",
        },
    ),
    RuleTest(
        name="nonetype error",
        expected_result=False,
        log={
            "XForwardedFor": ["1.2.3.4", "5.6.7.8"],
            "actionDescription": "Adds a new User role to Panther",
            "actionName": "CREATE_USER_ROLE",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "logTypeAccess": ["Okta.SystemLog"],
                        "logTypeAccessKind": "ALLOW",
                        "name": "ITE Role",
                        "permissions": ["AlertRead", "DataAnalyticsRead"],
                    },
                },
                "static": {},
            },
            "actionResult": "FAILED",
            "actor": {
                "attributes": {
                    "email": "random@noreply.com",
                    "emailVerified": False,
                    "roleId": "2a7bfe22-666d-4f71-99d2-c16b8666eca1",
                    "roleName": "Admin",
                },
                "id": "PantherSSO_random@noreply.com",
                "name": "random@noreply.com",
                "type": "USER",
            },
            "errors": [
                {
                    "message": "You cannot save a role that has both log type restrictions and alerts/detections permissions at this time.",
                },
            ],
            "p_alert_creation_time": "2023-02-09 21:47:09.745566000",
            "p_alert_id": "7eb5ca596b2153f95885cb2440e12345",
            "p_alert_severity": "HIGH",
            "p_alert_update_time": "2023-02-09 21:47:09.745566000",
            "p_any_ip_addresses": ["1.2.3.4", "5.6.7.8"],
            "p_any_trace_ids": ["PantherSSO_random@noreply.com"],
            "p_any_usernames": ["random@noreply.com"],
            "p_enrichment": {
                "ipinfo_asn": {
                    "sourceIP": {
                        "asn": "AS396982",
                        "domain": "google.com",
                        "name": "Google LLC",
                        "route": "208.127.224.0/21",
                        "type": "hosting",
                    },
                },
                "ipinfo_location": {
                    "sourceIP": {
                        "city": "Ashburn",
                        "country": "US",
                        "lat": "39.04372",
                        "lng": "-77.48749",
                        "postal_code": "20147",
                        "region": "Virginia",
                        "region_code": "VA",
                        "timezone": "America/New_York",
                    },
                },
            },
            "p_event_time": "2023-02-09 21:45:59.352910070",
            "p_log_type": "Panther.Audit",
            "p_parse_time": "2023-02-09 21:46:53.858602089",
            "p_row_id": "b29dff36ad73cb77a5d7a3a816c39c2a",
            "p_rule_error": '\'NoneType\' object is not iterable: Panther.Sensitive.Role.py, line 20, in rule    role_permissions = set(deep_get(event, "actionParams", "input", "permissions"))',
            "p_rule_id": "Panther.Sensitive.Role",
            "p_rule_reports": {"MITRE ATT&CK": ["TA0003:T1098"]},
            "p_rule_severity": "HIGH",
            "p_rule_tags": ["DataModel", "Persistence:Account Manipulation"],
            "p_schema_version": 0,
            "p_source_id": "9a116557-0a1c-4a21-8565-1135dfe5e82b",
            "p_source_label": "panther-audit-logs-us-east-1",
            "pantherVersion": "1.53.7",
            "sourceIP": "1.2.3.4",
            "timestamp": "2023-02-09 21:45:59.352910070",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
        },
    ),
]


@panther_managed
class PantherSensitiveRole(Rule):
    id = "Panther.Sensitive.Role-prototype"
    display_name = "A User Role with Sensitive Permissions has been Created"
    log_types = [LogType.PANTHER_AUDIT]
    default_severity = Severity.HIGH
    tags = ["DataModel", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_description = "A Panther user role has been created that contains admin level permissions."
    default_runbook = "Contact the creator of this role to ensure its creation was appropriate."
    default_reference = "https://docs.panther.com/system-configuration/rbac"
    summary_attributes = ["p_any_ip_addresses"]
    tests = panther_sensitive_role_tests
    PANTHER_ADMIN_PERMISSIONS = [
        "UserModify",
        "OrganizationAPITokenModify",
        "OrganizationAPITokenRead",
        "GeneralSettingsModify",
    ]
    PANTHER_ROLE_ACTIONS = [event_type.USER_GROUP_CREATED, event_type.USER_GROUP_MODIFIED]

    def rule(self, event):
        if event.udm("event_type") not in self.PANTHER_ROLE_ACTIONS:
            return False
        permissions = deep_get(event, "actionParams", "dynamic", "input", "permissions")
        if permissions is None:
            deep_get(event, "actionParams", "input", "permissions", default="")
        role_permissions = set(permissions)
        return (
            len(set(self.PANTHER_ADMIN_PERMISSIONS).intersection(role_permissions)) > 0
            and event.get("actionResult") == "SUCCEEDED"
        )

    def title(self, event):
        role_name = deep_get(event, "actionParams", "dynamic", "input", "name")
        if role_name is None:
            role_name = deep_get(event, "actionParams", "input", "name", default="<UNKNWON ROLE>")
        return f"Role with Admin Permissions created by {event.udm('actor_user')}Role Name: {role_name}"

    def alert_context(self, event):
        return {
            "user": event.udm("actor_user"),
            "role_name": deep_get(event, "actionParams", "name"),
            "ip": event.udm("source_ip"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

asana_workspace_require_app_approvals_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Web Reqs On",
        expected_result=False,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "1234",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:29:34.968",
            "details": {"new_value": "all_apps", "old_value": "off"},
            "event_category": "admin_settings",
            "event_type": "workspace_require_app_approvals_of_type_changed",
            "gid": "1234",
            "resource": {"gid": "1234", "name": "Panther Labs", "resource_type": "workspace"},
        },
    ),
    RuleTest(
        name="Web Reqs Off",
        expected_result=True,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "1234",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:29:34.968",
            "details": {"new_value": "off", "old_value": "all_apps"},
            "event_category": "admin_settings",
            "event_type": "workspace_require_app_approvals_of_type_changed",
            "gid": "1234",
            "resource": {"gid": "1234", "name": "Panther Labs", "resource_type": "workspace"},
        },
    ),
]


@panther_managed
class AsanaWorkspaceRequireAppApprovalsDisabled(Rule):
    default_description = (
        "An Asana user turned off app approval requirements for an application type for your organization."
    )
    display_name = "Asana Workspace Require App Approvals Disabled"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://help.asana.com/hc/en-us/articles/14109494654875-Admin-console#:~:text=used%20by%20default-,Require%20app%20approval,-Admins%20manage%20a"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.Require.App.Approvals.Disabled-prototype"
    tests = asana_workspace_require_app_approvals_disabled_tests

    def rule(self, event):
        new_val = deep_get(event, "details", "new_value", default="<NEW_VAL_NOT_FOUND>")
        return all(
            [
                event.get("event_type", "<NO_EVENT_TYPE_FOUND>") == "workspace_require_app_approvals_of_type_changed",
                new_val == "off",
            ],
        )

    def title(self, event):
        actor_email = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        context = deep_get(event, "context", "context_type", default="<APP_CONTEXT_NOT_FOUND>")
        return (
            f"Asana user [{actor_email}] disabled application approval requirements for [{context}] type applications."
        )



import os
import sys
from importlib.util import module_from_spec, spec_from_file_location


class NoMainModuleError(Exception):
    """NoMainModuleError is raised when no main module is found."""


def import_main(target_code_location: str = ".", main_module_name: str = "main"):
    """Imports the main module from the target code location. Assumes `setup` was called first."""
    customer_main_file = main_module_name + ".py"

    path = os.path.join(target_code_location, customer_main_file)
    if not os.path.isfile(path):
        raise NoMainModuleError(f"No {customer_main_file} found")

    sys.path.append(target_code_location)

    spec = spec_from_file_location(main_module_name, path)
    if spec is None:
        raise RuntimeError(f"No spec found for module={main_module_name} and path={path}")
    if spec.loader is None:
        raise RuntimeError(f"Spec has no loader for module={main_module_name} and path={path}")

    module = module_from_spec(spec)
    sys.modules[main_module_name] = module
    spec.loader.exec_module(module)



import datetime
import json
import time

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_ipinfo_helpers import IPInfoLocation
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.helpers.panther_oss_helpers import get_dictionary, put_dictionary

notion_login_from_new_location_tests: list[RuleTest] = [
    RuleTest(
        name="Login from normal location",
        expected_result=False,
        mocks=[
            RuleMock(object_name="get_dictionary", return_value='{ "Minas Tirith_Pellenor_Gondor": 1686542031 }'),
            RuleMock(object_name="put_dictionary", return_value=False),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Minas Tirith",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Gondor",
                        "postal_code": "55555",
                        "region": "Pellenor",
                        "region_code": "PL",
                        "timezone": "Middle Earth/Pellenor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="No previous recorded login",
        expected_result=False,
        mocks=[
            RuleMock(object_name="get_dictionary", return_value=""),
            RuleMock(object_name="put_dictionary", return_value=False),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Minas Tirith",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Gondor",
                        "postal_code": "55555",
                        "region": "Pellenor",
                        "region_code": "PL",
                        "timezone": "Middle Earth/Pellenor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Login from different location",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_dictionary", return_value='{ "Minas Tirith_Pellenor_Gondor": 1686542031 }'),
            RuleMock(object_name="put_dictionary", return_value=False),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Missing enrichment",
        expected_result=False,
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {},
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Unrelated event",
        expected_result=False,
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "page.viewed",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Login from different location - no region",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_dictionary", return_value='{ "Minas Tirith_Pellenor_Gondor": 1686542031 }'),
            RuleMock(object_name="put_dictionary", return_value=False),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
]


@panther_managed
class NotionLoginFromNewLocation(Rule):
    id = "Notion.LoginFromNewLocation-prototype"
    display_name = "Notion Login from New Location"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Identity & Access Management", "Login & Access Patterns"]
    default_severity = Severity.MEDIUM
    default_description = "A Notion User logged in from a new location."
    default_runbook = "Possible account takeover. Follow up with the Notion User to determine if this login is genuine."
    default_reference = "https://ipinfo.io/products/ip-geolocation-api"
    tests = notion_login_from_new_location_tests
    # How long (in seconds) to keep previous login locations in cached memory
    DEFAULT_CACHE_PERIOD = 2419200

    def rule(self, event):
        # Only focused on login events
        if event.deep_walk("event", "type") != "user.login":
            return False
        # Get the user's location, via IPInfo
        # Return False if we have no location information
        if "ipinfo_location" not in event.get("p_enrichment", {}):
            return False
        self.IPINFO_LOC = IPInfoLocation(event)
        path_to_ip = "event.ip_address"
        city = self.IPINFO_LOC.city(path_to_ip) or ""
        region = self.IPINFO_LOC.region(path_to_ip) or ""
        country = self.IPINFO_LOC.country(path_to_ip) or ""
        loc_string = "_".join((city, region, country))
        # Store the login location. The premise is to create a new entry for each combimation of user
        # and location, and then have those records persist for some length of time (4 weeks by
        # default).
        # Store the login location. Here, we use Panther's cache to store a dictionary, using the
        #   user's unique ID to ensure it hold data unique to them. In this dictionary, we'll use the
        #   location strings (loc_string) as the key, and the values will be the timestamp of the last
        #   recorded login from that location.
        user = event.deep_walk("event", "actor", "id")
        cache = get_dictionary(user) or {}
        # If this is a unit test, convert cache from string
        if isinstance(cache, str):
            cache = json.loads(cache)
        # -- Step 1: Record this login.
        new_cache = cache.copy()
        new_cache[loc_string] = time.time()
        put_dictionary(user, new_cache)
        # -- Step 2: Determine if we shoul raise an alert.
        if not cache:
            # User hasn't been recorded logging in before. Since this is their first login, we don't
            #   have a baseline to know if it's unusual, so we won't raise an alert.
            return False
        if self.is_recent_login(cache, loc_string, event.get("p_parse_time")):
            # User has logged in from this location in the recent past. No need to raise an alert.
            return False
        # User has NOT logged in from this location in the recent past - we should trigger an alert!
        return True

    def title(self, event):
        path_to_ip = "event.ip_address"
        city = self.IPINFO_LOC.city(path_to_ip)
        region = self.IPINFO_LOC.region(path_to_ip)
        country = self.IPINFO_LOC.country(path_to_ip)
        user_email = event.deep_walk("event", "actor", "person", "email", default="UNKNWON_EMAIL")
        return f"Notion [{user_email}] logged in from a new location: {city}, {region}, {country}."

    def alert_context(self, event):
        path_to_ip = "event.ip_address"
        city = self.IPINFO_LOC.city(path_to_ip)
        region = self.IPINFO_LOC.region(path_to_ip)
        country = self.IPINFO_LOC.country(path_to_ip)
        user_email = event.deep_walk("event", "actor", "person", "email", default="UNKNWON_EMAIL")
        context = notion_alert_context(event)
        context["user_email"] = user_email
        context["location"] = {"city": city, "region": region, "country": country}
        return context

    def is_recent_login(self, cache: dict, loc_string: str, parse_time: str) -> bool:
        # Use p_parse_time to calculate current timestamp, so that unit tests work.
        now = time.mktime(
            datetime.datetime.fromisoformat(parse_time[:23]).timetuple(),
        )  # location was previously recorded
        # last recorded login is recent
        return loc_string in cache and cache[loc_string] > now - self.DEFAULT_CACHE_PERIOD



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

aws_cloud_trail_user_access_key_auth_tests: list[RuleTest] = [
    RuleTest(
        name="Access Key Action Failed",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::187901811700:user/exposed.user is not authorized to perform: lambda:ListFunctions on resource: * because no identity-based policy allows the lambda:ListFunctions action",
            "eventCategory": "Management",
            "eventID": "57b719ad-f2fa-4d18-bd84-29fee11799b1",
            "eventName": "ListFunctions20150331",
            "eventSource": "lambda.amazonaws.com",
            "eventTime": "2024-06-02 20:16:32.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_actor_ids": ["AIDASXP6SDP2AJQPWVFII"],
            "p_any_aws_account_ids": ["187901811700"],
            "p_any_aws_arns": ["arn:aws:iam::187901811700:user/exposed.user"],
            "p_any_ip_addresses": ["73.252.165.138"],
            "p_any_trace_ids": ["AKIASXP6SDP2F3JQERZ2"],
            "p_any_usernames": ["exposed.user"],
            "p_event_time": "2024-06-02 20:16:32.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2024-06-02 20:25:54.290533382",
            "p_row_id": "f26228572e3f9acd88e0cadc1fcc8e0a",
            "p_schema_version": 0,
            "p_source_file": {
                "aws_s3_bucket": "threat-research-trail-trail-bucket-0ipb5nzxam",
                "aws_s3_key": "AWSLogs/187901811700/CloudTrail/us-east-1/2024/06/02/187901811700_CloudTrail_us-east-1_20240602T2020Z_12tLndRWeXi4IWKg.json.gz",
            },
            "p_source_id": "d0a1e235-6548-4e7f-952a-35063b304007",
            "p_source_label": "threat-research-trail-us-east-1",
            "p_udm": {
                "source": {"address": "73.252.165.138", "ip": "73.252.165.138"},
                "user": {"arns": ["arn:aws:iam::187901811700:user/exposed.user"], "name": "exposed.user"},
            },
            "readOnly": True,
            "recipientAccountId": "187901811700",
            "requestID": "ea7dbdf9-6c03-421e-abf4-f201919f9f26",
            "sourceIPAddress": "73.252.165.138",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "lambda.us-east-1.amazonaws.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "aws-cli/2.15.59 md/awscrt#0.19.19 ua/2.0 os/macos#22.6.0 md/arch#arm64 lang/python#3.11.9 md/pyimpl#CPython cfg/retry-mode#standard md/installer#source md/prompt#off md/command#lambda.list-functions",
            "userIdentity": {
                "accessKeyId": "AKIASXP6SDP2F3JQERZ2",
                "accountId": "187901811700",
                "arn": "arn:aws:iam::187901811700:user/exposed.user",
                "principalId": "AIDASXP6SDP2AJQPWVFII",
                "type": "IAMUser",
                "userName": "exposed.user",
            },
        },
    ),
    RuleTest(
        name="Access Key Action Success",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "4c24450d-007e-4849-9e1b-4954622dbb08",
            "eventName": "GetCallerIdentity",
            "eventSource": "sts.amazonaws.com",
            "eventTime": "2024-06-02 20:16:22.000000000",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_actor_ids": ["AIDASXP6SDP2AJQPWVFII"],
            "p_any_aws_account_ids": ["187901811700"],
            "p_any_aws_arns": ["arn:aws:iam::187901811700:user/exposed.user"],
            "p_any_ip_addresses": ["73.252.165.138"],
            "p_any_trace_ids": ["AKIASXP6SDP2F3JQERZ2"],
            "p_any_usernames": ["exposed.user"],
            "p_event_time": "2024-06-02 20:16:22.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2024-06-02 20:25:54.289981899",
            "p_row_id": "f26228572e3f9acd88e0cadc1fc88e0a",
            "p_schema_version": 0,
            "p_source_file": {
                "aws_s3_bucket": "threat-research-trail-trail-bucket-0ipb5nzxam",
                "aws_s3_key": "AWSLogs/187901811700/CloudTrail/us-east-1/2024/06/02/187901811700_CloudTrail_us-east-1_20240602T2020Z_12tLndRWeXi4IWKg.json.gz",
            },
            "p_source_id": "d0a1e235-6548-4e7f-952a-35063b304007",
            "p_source_label": "threat-research-trail-us-east-1",
            "p_udm": {
                "source": {"address": "73.252.165.138", "ip": "73.252.165.138"},
                "user": {"arns": ["arn:aws:iam::187901811700:user/exposed.user"], "name": "exposed.user"},
            },
            "readOnly": True,
            "recipientAccountId": "187901811700",
            "requestID": "c77ed3ee-8480-41df-98cb-9cf52ce04a1c",
            "sourceIPAddress": "73.252.165.138",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "sts.us-east-1.amazonaws.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "aws-cli/2.15.59 md/awscrt#0.19.19 ua/2.0 os/macos#22.6.0 md/arch#arm64 lang/python#3.11.9 md/pyimpl#CPython cfg/retry-mode#standard md/installer#source md/prompt#off md/command#sts.get-caller-identity",
            "userIdentity": {
                "accessKeyId": "AKIASXP6SDP2F3JQERZ2",
                "accountId": "187901811700",
                "arn": "arn:aws:iam::187901811700:user/exposed.user",
                "principalId": "AIDASXP6SDP2AJQPWVFII",
                "type": "IAMUser",
                "userName": "exposed.user",
            },
        },
    ),
    RuleTest(
        name="Console Login",
        expected_result=False,
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "364ad368-42bf-4e05-a500-971ddfe8ebff",
            "eventName": "ConsoleLogin",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2024-06-02 19:41:40.000000000",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_actor_ids": ["AROASXP6SDP2F4WLQVARB", "AROASXP6SDP2F4WLQVARB:nicholas.hakmiller"],
            "p_any_aws_account_ids": ["187901811700"],
            "p_any_aws_arns": [
                "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_DevAdmin_635426549a280cc6",
                "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
            ],
            "p_any_ip_addresses": ["73.252.165.138"],
            "p_any_trace_ids": ["ASIASXP6SDP2HUZY3TOB"],
            "p_any_usernames": ["AWSReservedSSO_DevAdmin_635426549a280cc6", "nicholas.hakmiller"],
            "p_event_time": "2024-06-02 19:41:40.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2024-06-02 19:50:54.391407154",
            "p_row_id": "f26228572e3f9acd88e0cadc1f80a502",
            "p_schema_version": 0,
            "p_source_file": {
                "aws_s3_bucket": "threat-research-trail-trail-bucket-xhh4yndpq5",
                "aws_s3_key": "AWSLogs/187901811700/CloudTrail/us-west-2/2024/06/02/187901811700_CloudTrail_us-west-2_20240602T1945Z_ggfzMNc1AHPJypqR.json.gz",
            },
            "p_source_id": "469edf86-a0c6-4d13-ba48-47c4060bb804",
            "p_source_label": "threat-research-trail-us-west-2",
            "p_udm": {
                "source": {"address": "73.252.165.138", "ip": "73.252.165.138"},
                "user": {
                    "arns": [
                        "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_DevAdmin_635426549a280cc6",
                        "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                    ],
                },
            },
            "readOnly": False,
            "recipientAccountId": "187901811700",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "73.252.165.138",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "us-west-2.signin.aws.amazon.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "ASIASXP6SDP2HUZY3TOB",
                "accountId": "187901811700",
                "arn": "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                "principalId": "AROASXP6SDP2F4WLQVARB:nicholas.hakmiller",
                "sessionContext": {
                    "attributes": {"creationDate": "2024-06-02T19:41:40Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "187901811700",
                        "arn": "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
                        "principalId": "AROASXP6SDP2F4WLQVARB",
                        "type": "Role",
                        "userName": "AWSReservedSSO_DevAdmin_635426549a280cc6",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="No Access Key",
        expected_result=False,
        log={
            "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "364ad368-42bf-4e05-a500-971ddfe8ebff",
            "eventName": "AssumeRole",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2024-06-02 19:41:40.000000000",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_actor_ids": ["AROASXP6SDP2F4WLQVARB", "AROASXP6SDP2F4WLQVARB:nicholas.hakmiller"],
            "p_any_aws_account_ids": ["187901811700"],
            "p_any_aws_arns": [
                "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_DevAdmin_635426549a280cc6",
                "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/us-west-2/AWSReservedSSO_DevAdmin_635426549a280cc6",
            ],
            "p_any_ip_addresses": ["73.252.165.138"],
            "p_any_trace_ids": ["ASIASXP6SDP2HUZY3TOB"],
            "p_any_usernames": ["nicholas.hakmiller"],
            "p_event_time": "2024-06-02 19:41:40.000000000",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2024-06-02 19:50:54.391407154",
            "p_row_id": "f26228572e3f9acd88e0cadc1f80a502",
            "p_schema_version": 0,
            "p_source_file": {
                "aws_s3_bucket": "threat-research-trail-trail-bucket-xhh4yndpq5",
                "aws_s3_key": "AWSLogs/187901811700/CloudTrail/us-west-2/2024/06/02/187901811700_CloudTrail_us-west-2_20240602T1945Z_ggfzMNc1AHPJypqR.json.gz",
            },
            "p_source_id": "469edf86-a0c6-4d13-ba48-47c4060bb804",
            "p_source_label": "threat-research-trail-us-west-2",
            "p_udm": {
                "source": {"address": "73.252.165.138", "ip": "73.252.165.138"},
                "user": {
                    "arns": [
                        "arn:aws:iam::187901811700:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_DevAdmin_635426549a280cc6",
                        "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                    ],
                },
            },
            "readOnly": False,
            "recipientAccountId": "187901811700",
            "responseElements": {"ConsoleLogin": "Success"},
            "sourceIPAddress": "73.252.165.138",
            "tlsDetails": {
                "cipherSuite": "TLS_AES_128_GCM_SHA256",
                "clientProvidedHostHeader": "us-west-2.signin.aws.amazon.com",
                "tlsVersion": "TLSv1.3",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36",
            "userIdentity": {
                "accountId": "187901811700",
                "arn": "arn:aws:sts::187901811700:assumed-role/AWSReservedSSO_DevAdmin_635426549a280cc6/nicholas.hakmiller",
                "principalId": "AROASXP6SDP2F4WLQVARB:nicholas.hakmiller",
                "sessionContext": {
                    "attributes": {"creationDate": "2024-06-02T19:41:40Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {"type": "lambda"},
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailUserAccessKeyAuth(Rule):
    id = "AWS.CloudTrail.UserAccessKeyAuth-prototype"
    display_name = "AWS.CloudTrail.UserAccessKeyAuth"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    inline_filters = [{"All": []}]
    create_alert = False
    tests = aws_cloud_trail_user_access_key_auth_tests

    def rule(self, event):
        # Only look for successes
        if event.get("errorCode") or event.get("errorMessage"):
            return False
        # Reference: https://awsteele.com/blog/2020/09/26/aws-access-key-format.html
        return event.deep_get("userIdentity", "accessKeyId", default="").startswith("AKIA")

    def title(self, event):
        arn = event.deep_get("userIdentity", "arn")
        key = event.deep_get("userIdentity", "accessKeyId")
        return f"User {arn} signed in with access key {key}"

    def alert_context(self, event):
        return {
            "ip_accessKeyId": event.get("sourceIpAddress", default="{not found}")
            + ":"
            + event.deep_get("userIdentity", "accessKeyId", default="{not found}"),
        }



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context

okta_admin_role_assigned_tests: list[RuleTest] = [
    RuleTest(
        name="Admin Access Assigned",
        expected_result=True,
        log={
            "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
            "published": "2020-11-25 21:27:03.496000000",
            "eventType": "user.account.privilege.grant",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.admin_privilege.granted",
            "displayMessage": "Grant user privilege",
            "actor": {
                "id": "00uu1uuuuIlllaaaa356",
                "type": "User",
                "alternateId": "jack@acme.io",
                "displayName": "Jack Naglieri",
            },
            "client": {
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36",
                },
                "geographicalContext": {
                    "geolocation": {"lat": 37.7852, "lon": -122.3874},
                    "city": "San Francisco",
                    "state": "California",
                    "country": "United States",
                    "postalCode": "94105",
                },
                "zone": "null",
                "ipAddress": "136.24.229.58",
                "device": "Computer",
            },
            "request": {},
            "outcome": {"result": "SUCCESS"},
            "target": [
                {
                    "id": "00u6eup97mAJZWYmP357",
                    "type": "User",
                    "alternateId": "alice@acme.io",
                    "displayName": "Alice Green",
                },
            ],
            "transaction": {},
            "debugContext": {
                "debugData": {
                    "privilegeGranted": "Organization administrator, Application administrator (all)",
                    "requestUri": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                    "threatSuspected": "false",
                    "url": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                    "requestId": "X777JJ9sssQQHHrrrQTyYQAABBE",
                },
            },
            "authenticationContext": {},
            "securityContext": {},
        },
    ),
    RuleTest(
        name="Super Admin Access Assigned (High sev)",
        expected_result=True,
        log={
            "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
            "published": "2020-11-25 21:27:03.496000000",
            "eventType": "user.account.privilege.grant",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.admin_privilege.granted",
            "displayMessage": "Grant user privilege",
            "actor": {
                "id": "00uu1uuuuIlllaaaa356",
                "type": "User",
                "alternateId": "jack@acme.io",
                "displayName": "Jack Naglieri",
            },
            "client": {
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36",
                },
                "geographicalContext": {
                    "geolocation": {"lat": 37.7852, "lon": -122.3874},
                    "city": "San Francisco",
                    "state": "California",
                    "country": "United States",
                    "postalCode": "94105",
                },
                "zone": "null",
                "ipAddress": "136.24.229.58",
                "device": "Computer",
            },
            "request": {},
            "outcome": {"result": "SUCCESS"},
            "target": [
                {
                    "id": "00u6eup97mAJZWYmP357",
                    "type": "User",
                    "alternateId": "alice@acme.io",
                    "displayName": "Alice Green",
                },
            ],
            "transaction": {},
            "debugContext": {
                "debugData": {
                    "privilegeGranted": "Super administrator, Read only admin",
                    "requestUri": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                    "threatSuspected": "false",
                    "url": "/api/internal/administrators/00u6eu8c68bb72a21b57",
                    "requestId": "X777JJ9sssQQHHrrrQTyYQAABBE",
                },
            },
            "authenticationContext": {},
            "securityContext": {},
        },
    ),
]


@panther_managed
class OktaAdminRoleAssigned(Rule):
    id = "Okta.AdminRoleAssigned-prototype"
    display_name = "Okta Admin Role Assigned"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Okta", "Privilege Escalation:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.INFO
    default_description = "A user has been granted administrative privileges in Okta"
    default_reference = "https://help.okta.com/en/prod/Content/Topics/Security/administrators-admin-comparison.htm"
    default_runbook = "Reach out to the user if needed to validate the activity"
    dedup_period_minutes = 15
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    tests = okta_admin_role_assigned_tests
    ADMIN_PATTERN = re.compile("[aA]dministrator")

    def rule(self, event):
        return (
            event.get("eventType", None) == "user.account.privilege.grant"
            and deep_get(event, "outcome", "result") == "SUCCESS"
            and bool(
                self.ADMIN_PATTERN.search(deep_get(event, "debugContext", "debugData", "privilegeGranted", default="")),
            )
        )

    def dedup(self, event):
        return deep_get(event, "debugContext", "debugData", "requestId", default="<UNKNOWN_REQUEST_ID>")

    def title(self, event):
        target = event.get("target", [{}])
        display_name = target[0].get("displayName", "MISSING DISPLAY NAME") if target else ""
        alternate_id = target[0].get("alternateId", "MISSING ALTERNATE ID") if target else ""
        privilege = deep_get(event, "debugContext", "debugData", "privilegeGranted", default="<UNKNOWN_PRIVILEGE>")
        return f"{deep_get(event, 'actor', 'displayName')} <{deep_get(event, 'actor', 'alternateId')}> granted [{privilege}] privileges to {display_name} <{alternate_id}>"

    def alert_context(self, event):
        return okta_alert_context(event)

    def severity(self, event):
        if "Super administrator" in deep_get(event, "debugContext", "debugData", "privilegeGranted", default=""):
            return "HIGH"
        return "INFO"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_role_created_tests: list[RuleTest] = [
    RuleTest(
        name="A role was created",
        expected_result=True,
        log={
            "cluster_name": "teleport.example.com",
            "code": "T9000I",
            "ei": 0,
            "event": "role.created",
            "expires": "0001-01-01T00:00:00Z",
            "name": "teleport-event-handler",
            "time": "2023-09-20T23:00:000.000000Z",
            "uid": "88888888-4444-4444-4444-222222222222",
            "user": "max.mustermann@example.com",
        },
    ),
]


@panther_managed
class TeleportRoleCreated(Rule):
    id = "Teleport.RoleCreated-prototype"
    display_name = "A Teleport Role was modified or created"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["Teleport"]
    default_severity = Severity.MEDIUM
    default_description = "A Teleport Role was modified or created"
    reports = {"MITRE ATT&CK": ["TA0003:T1098.001"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "A Teleport Role was modified or created. Validate its legitimacy.\n"
    summary_attributes = ["event", "code", "user", "name"]
    tests = teleport_role_created_tests

    def rule(self, event):
        return event.get("event") == "role.created"

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] created Role [{event.get('name', '<UNKNOWN_NAME>')}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_audit_user_added_outside_org_tests: list[RuleTest] = [
    RuleTest(
        name="Outside org",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "Added user badguy@acme.io to org 12345 (Email Invitation)",
            "eventId": "d109e568832111ee8ab2057b240e65f8",
            "eventTime": "2023-11-14 19:12:55.917000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Inside org",
        expected_result=False,
        log={
            "clientIp": "12.34.56.78",
            "description": "Added user goodguy@acme.com to org 12345 (Email Invitation)",
            "eventId": "d109e568832111ee8ab2057b240e65f8",
            "eventTime": "2023-11-14 19:12:55.917000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
]


@panther_managed
class CarbonBlackAuditUserAddedOutsideOrg(Rule):
    id = "CarbonBlack.Audit.User.Added.Outside.Org-prototype"
    log_types = [LogType.CARBON_BLACK_AUDIT]
    default_description = "Detects when a user from a different organization is added to Carbon Black."
    display_name = "Carbon Black User Added Outside Org"
    default_severity = Severity.HIGH
    tags = ["Persistence", "Create Account"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-516BAF8C-A13D-4FC7-AA92-923159C13083.html"
    tests = carbon_black_audit_user_added_outside_org_tests
    PATTERNS = ("Added user ",)

    def rule(self, event):
        desc = event.get("description", "")
        if not any(desc.startswith(pattern) for pattern in self.PATTERNS):
            return False
        src_user = event.get("loginName", "")
        src_domain = src_user.split("@")[1]
        dst_user = desc.split(" ")[2]
        dst_domain = dst_user.split("@")[1]
        if src_domain != dst_domain:
            return True
        return False

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"
